import { NullEmptyOrWhitespace } from "../src/guard/catch/NullEmptyOrWhitespace";
import { ValidNumber, EqualObjects } from "../src/guard/ensure";

describe("EqualObjects", () => {
    it("should return true for equal objects", () => {
        const obj1 = { name: "John", age: 25 };
        const obj2 = { name: "John", age: 25 };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(true);
    });

    it("should return false for different objects", () => {
        const obj1 = { name: "John", age: 25 };
        const obj2 = { name: "Jane", age: 25 };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });

    it("should return false for objects with different number of properties", () => {
        const obj1 = { name: "John", age: 25 };
        const obj2 = { name: "John", age: 25, city: "New York" };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });

    it("should return false for objects with different property values", () => {
        const obj1 = { name: "John", age: 25 };
        const obj2 = { name: "John", age: 30 };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });

    it("should return true for equal objects with nested objects", () => {
        const obj1 = { name: "John", address: { city: "New York", state: "NY" } };
        const obj2 = { name: "John", address: { city: "New York", state: "NY" } };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(true);
    });

    it("should return false for objects with different nested objects", () => {
        const obj1 = { name: "John", address: { city: "New York", state: "NY" } };
        const obj2 = { name: "John", address: { city: "Los Angeles", state: "CA" } };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });

    it("should return false for objects with different nested object values", () => {
        const obj1 = { name: "John", address: { city: "New York", state: "NY" } };
        const obj2 = { name: "John", address: { city: "New York", state: "CA" } };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });

    it("should return true for equal objects with nested arrays", () => {
        const obj1 = { name: "John", hobbies: ["reading", "swimming"] };
        const obj2 = { name: "John", hobbies: ["reading", "swimming"] };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(true);
    });

    it("should return false for objects with different nested arrays", () => {
        const obj1 = { name: "John", hobbies: ["reading", "swimming"] };
        const obj2 = { name: "John", hobbies: ["reading", "running"] };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });

    it("should return false for objects with different nested array lengths", () => {
        const obj1 = { name: "John", hobbies: ["reading", "swimming"] };
        const obj2 = { name: "John", hobbies: ["reading", "swimming", "running"] };
        const result = EqualObjects(obj1, obj2);
        expect(result).toBe(false);
    });
});

describe("NullEmptyOrWhitespace", () => {
    it("should return true for null input", () => expect(NullEmptyOrWhitespace(null)).toBe(true));
    it("should return true for undefined input", () => expect(NullEmptyOrWhitespace(undefined)).toBe(true));
    it("should return true for empty string input", () => expect(NullEmptyOrWhitespace("")).toBe(true));
    it("should return true for string with only whitespace input", () => expect(NullEmptyOrWhitespace("   ")).toBe(true));
    it("should return true for empty array input", () => expect(NullEmptyOrWhitespace([])).toBe(true));
    it("should return true for empty object input", () => expect(NullEmptyOrWhitespace({})).toBe(true));
    it("should return true for empty Set input", () => expect(NullEmptyOrWhitespace(new Set())).toBe(true));
    it("should return true for empty Map input", () => expect(NullEmptyOrWhitespace(new Map())).toBe(true));
    it("should return false for non-empty input", () => {
        expect(NullEmptyOrWhitespace("hello")).toBe(false);
        expect(NullEmptyOrWhitespace([1, 2, 3])).toBe(false);
        expect(NullEmptyOrWhitespace({ key: "value" })).toBe(false);
        expect(NullEmptyOrWhitespace(new Set([1, 2, 3]))).toBe(false);
        expect(
            NullEmptyOrWhitespace(
                new Map([
                    [1, "one"],
                    [2, "two"],
                ])
            )
        ).toBe(false);
    });
});

describe("ValidNumber", () => {
    it("should return true when input is a valid number within the specified range", () => {
        expect(ValidNumber(5, 0, 10)).toBe(true);
        expect(ValidNumber(-2.5, -10, 0)).toBe(true);
        expect(ValidNumber("3.14", 0, 5)).toBe(true);
    });
    it("should return true when input is a valid number without specifying a range", () => {
        expect(ValidNumber(5)).toBe(true);
        expect(ValidNumber("-2.5")).toBe(true);
    });
    it("should return false when input is not a number", () => {
        expect(ValidNumber("abc")).toBe(false);
        expect(ValidNumber("")).toBe(false);
    });
    it("should return false when input is less than the specified minimum value", () => {
        expect(ValidNumber(-5, 0, 10)).toBe(false);
        expect(ValidNumber("-10", 0, 5)).toBe(false);
    });
    it("should return false when input is greater than the specified maximum value", () => {
        expect(ValidNumber(15, 0, 10)).toBe(false);
        expect(ValidNumber("7", -5, 5)).toBe(false);
    });
});
